# pylint: disable-all
"""Test file for cli.py
"""
from unittest import TestCase
import cerberus
from click.testing import CliRunner
from scheckcli.cli import main


class SetUp:
    """SetUp class for the  creates the click runner and reusuable data"""

    def click_runner(self, *args):
        """Creates a clirunner."""
        if args:
            self.runner = CliRunner()
            self.result = self.runner.invoke(main, args)
        else:
            self.runner = CliRunner()
            self.result = self.runner.invoke(main)
        return (self.runner, self.result)

    def click_runner_isolated_fs(self, *args):
        self.runner = CliRunner()
        with self.runner.isolated_filesystem():
            with open('schema.yaml', 'w') as sfp:
                sfp.write(self.load_schema_text)
            with open('input.yaml', 'w') as ifp:
                ifp.write(self.load_input_text)
            self.result = self.runner.invoke(main, args)
            return (self.result, sfp, ifp)

    def compare_click_runner_isolate_fs(self):
        SetUp.click_runner_isolated_fs(
            self, "compare", "-sfp", "schema.yaml", "-ifp", "input.yaml"
        )

    def check_click_runner_isolate_fs(self):
        SetUp.click_runner_isolated_fs(
            self, "check", "-sfp", "schema.yaml"
        )

    def base_assert(self):
        assert self.result.exit_code == 0
        assert "--help" in self.result.output
        assert "Show this message and exit." in self.result.output

    def valid_data(self):
        self.load_schema_text = """
            spec:
              type: dict
              required: true
              schema:
                name:
                  type: string
                  required: true
                feeds:
                  type: list
                  required: true
                  schema:
                    type: dict
                    schema:
                      name:
                        type: string
                        required: true
                      feedID:
                        type: string
                        required: true
        """
        self.load_input_text = """
            spec:
              name: testName
              feeds:
                - name: foo
                  feedID: bar
                - name: spam
                  feedID: eggs
        """
        return (self.load_schema_text, self.load_input_text)

    def invalid_data(self):
        self.load_schema_text = """
            spec:
        """
        self.load_input_text = """
            name:
        """
        return (self.load_schema_text, self.load_input_text)


class test_base(TestCase):
    def test_no_params(self):
        SetUp.click_runner(self)
        SetUp.base_assert(self)

    def test_compare_no_params(self):
        SetUp.click_runner(self, "compare")
        SetUp.base_assert(self)

    def test_check_no_params(self):
        SetUp.click_runner(self, "check")
        SetUp.base_assert(self)


class test_compare(TestCase):
    def test_no_ifp(self):
        SetUp.click_runner(
            self, "compare", "--schema-file-path", "schema.yaml"
        )
        assert self.result.exit_code == 2
        assert (
            "Error: Missing option '--input-file-path' / '-ifp'."
            in self.result.output
        )

    def test_no_sfp_input(self):
        SetUp.click_runner(self, "compare", "-ifp", "input.yaml")
        assert self.result.exit_code == 2
        assert (
            "Error: Missing option '--schema-file-path' / '-sfp'."
            in self.result.output
        )

    def test_missing_file(self):
        SetUp.click_runner(
            self, "compare", "-sfp", "schema.null", "-ifp", "input.null"
        )
        assert self.result.exit_code == 2
        assert (
            "Path 'schema.null' does not exist." in self.result.output
        )

    def test_schemafile_yaml_error(self):
        SetUp.valid_data(self)[1]
        SetUp.invalid_data(self)[0]
        SetUp.compare_click_runner_isolate_fs(self)
        assert self.result.exit_code == 1
        assert isinstance(self.result.exception, TypeError)

    def test_input_with_invalid_schema(self):
        SetUp.valid_data(self)[0]
        self.load_input_text = """
          name:
        """
        SetUp.compare_click_runner_isolate_fs(self)
        assert self.result.exit_code == 1
        assert isinstance(self.result.exception, Exception)

    def test_valid_schema_pass(self):
        SetUp.valid_data(self)
        SetUp.compare_click_runner_isolate_fs(self)
        assert self.result.exit_code == 0
        assert 'Input file has a valid schema.' in self.result.output

    def test_empty_schema_pass(self):
        SetUp.valid_data(self)[1]
        self.load_schema_text = """"""
        SetUp.compare_click_runner_isolate_fs(self)
        assert self.result.exit_code == 1
        assert isinstance(self.result.exception, cerberus.SchemaError)


class test_check(TestCase):
    def test_valid_schema(self):
        SetUp.valid_data(self)[0]
        SetUp.check_click_runner_isolate_fs(self)
        assert self.result.exit_code == 0
        assert 'The Schema file has valid syntax.' in self.result.output

    def test_invalid_schema(self):
        SetUp.invalid_data(self)[1]
        SetUp.check_click_runner_isolate_fs(self)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8'),
    ) as fh:
        return fh.read()


setup(
    name='scheckcli',
    version='1.1.0',
    license='MIT',
    description='A cli tool to perform json and yaml object schema validation.',
    long_description='{}\n{}'.format(
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub(
            '', read('README.rst')
        ),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst')),
    ),
    author='Ron Mallory',
    author_email='ronnymallory@gmail.com',
    url='https://gitlab.com/ronmallory/python-scheckcli',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[
        splitext(basename(path))[0] for path in glob('src/*.py')
    ],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        # uncomment if you test on these interpreters:
        # 'Programming Language :: Python :: Implementation :: IronPython',
        # 'Programming Language :: Python :: Implementation :: Jython',
        # 'Programming Language :: Python :: Implementation :: Stackless',
        'Topic :: Utilities',
    ],
    project_urls={
        'Documentation': 'https://python-scheckcli.readthedocs.io/',
        'Changelog': 'https://python-scheckcli.readthedocs.io/en/latest/changelog.html',
        'Issue Tracker': 'https://gitlab.com/ronmallory/python-scheckcli/issues',
    },
    keywords=[
        # eg: 'keyword1', 'keyword2', 'keyword3',
    ],
    python_requires='>=3.6',
    install_requires=['click', 'pyyaml', 'cerberus'],
    extras_require={
        # eg:
        #   'rst': ['docutils>=0.11'],
        #   ':python_version=="2.6"': ['argparse'],
    },
    entry_points={
        'console_scripts': [
            'scheckcli = scheckcli.cli:main',
        ]
    },
)



"""
Module that contains the command line app.

"""

from os import environ
import click
import yaml
from cerberus import Validator, SchemaError


class ProcessFiles:
    """Reads the input schema files

    Args:
        schema_file_path (filePath, required): file path to the schema file.
        input_file_path  (filePath, optional): file path to the schema file.
    Returns:

    """

    def __init__(self, schema_file_path, input_file_path=None):
        self.schema_file_path = schema_file_path
        self.input_file_path = input_file_path or {}
        self._get_input = None
        self._get_schema = None
        self.stream = None
        self.validate = None

    def get_yaml(self, file_path=None):
        """yaml.SafeLoads the file provided in file_path.

        Args:
            file_path (_type_, optional): _description_. Defaults to None.
        Raises:
            raises a yaml.YAMLError if the yaml syntax is invalid
        Returns:
            yaml Object:
        """
        with open(file_path, "r", encoding="utf-8") as self.stream:
            try:
                return yaml.load(self.stream, Loader=yaml.SafeLoader)
            except yaml.YAMLError as exception:
                return print(
                    "The file does not have a valid structure: ",
                    exception,
                )

    def get_schema(self):
        """_summary_

        Returns:
            _type_: _description_
        """
        if self._get_schema is None:
            self._get_schema = self.get_yaml(
                file_path=self.schema_file_path
            )
            self.validate = Validator(self._get_schema)
            return self.validate
        try:
            return (self._get_schema, self.validate)
        except TypeError as exception:
            return print("Invalid Schema file Exception: ", exception)

    def get_input(self):
        '''loads the input file'''
        if self._get_input is None:
            self._get_input = self.get_yaml(
                file_path=self.input_file_path
            )

    def compare(self):
        """
        compare_cmd validates the input file
        has the same schema as the schema file.

        Args:
            schema_file_path (file_path): File path of the schema file.
            input_file_path (file_path): File path of the input file.
        """
        self.get_schema()
        self.get_input()
        try:
            if self.validate.validate(self._get_input):
                return print('Input file has a valid schema.'), 0
            raise Exception(self.validate.errors)
        except SchemaError as exception:
            raise exception


@click.group()
def cli1():
    """create cli click group"""
    pass  # pylint: disable=W0107


@cli1.command(
    'compare',
    no_args_is_help=True,
    short_help="Compare schema of the input file against a schema file.",
    help="""\b
    Compare schema of the input file against a schema file.
    
    EXAMPLES:

        scheckcli compare -sfp ./schema.yaml -ifp./input.yaml
    
        scheckcli compare -sfp ./schema.json -ifp ./input.json
    """,
)
@click.option(
    '--schema-file-path',
    '-sfp',
    required=True,
    help="""
        The file path to the schema file.
        Configurable with SCHECK_SCHEMA_PATH environmental Variable""",
    default=lambda: environ.get("SCHECK_SCHEMA_PATH"),
    type=click.Path(exists=True, file_okay=True),
)
@click.option(
    '--input-file-path',
    '-ifp',
    required=True,
    help="""
    The file path to the input file to compare.
    Configurable with SCHECK_INPUT_PATH environmental variable.""",
    default=lambda: environ.get("SCHECK_INPUT_PATH"),
    type=click.Path(exists=True, file_okay=True),
)
def compare_cmd(schema_file_path, input_file_path):
    """
    compare_cmd validates the input file
    has the same schema as the schema file.

    Args:
        schema_file_path (file_path): File path of the schema file.
        input_file_path (file_path): File path of the input file.
    """
    comp = ProcessFiles(schema_file_path, input_file_path)
    comp.compare()


@click.group()
def cli2():
    """create cli2 click group"""
    pass  # pylint: disable=W0107


@cli2.command(
    'check',
    no_args_is_help=True,
    short_help="Checks the provided schema file for valid syntax.",
    help="""\b
    Checks the provided schema file for valid syntax. Both json or yaml are accepted file types.
    
    EXAMPLES:
    
        scheckcli check ./schema.yaml

        scheckcli check ./schema.json

        scheckcli check --sfp ./schema.yaml

        scheckcli check --sfp ./schema.json
    """,
)
@click.option(
    '--schema-file-path',
    '-sfp',
    required=True,
    help="""
    The file path to the schema file.
    Configurable with SCHECK_SCHEMA_PATH environmental Variable""",
    default=lambda: environ.get("SCHECK_SCHEMA_PATH"),
    type=click.Path(exists=True, file_okay=True),
)
def check_command(schema_file_path):
    """check_command ensure schema file is valid.

    Args:
        schema_file_path (filePath): The file path of the schema file.
    """
    comp = ProcessFiles(schema_file_path)
    if comp.get_schema():
        print("The Schema file has valid syntax.")


main = click.CommandCollection(
    sources=[cli1, cli2],
    help="""
        Compare given input file with given schema file. Check's the provided schema file for valid schema syntax.
        """,
)
